package de.ozgcloud.alfa.collaboration;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Optional;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class OrganisationsEinheitVorgangWithEingangProcessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_SEARCH_ORGANISATIONS_EINHEIT = LinkRelation.of("searchOrganisationsEinheit");

	private final FeatureToggleProperties featureToggleProperties;
	private final CurrentUserService userService;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		Optional.ofNullable(model.getContent())
				.ifPresent(vorgang -> model.addIf(isSearchNeeded(vorgang),
						this::buildSearchOrganisationsEinheitLink));
		return model;
	}

	boolean isSearchNeeded(VorgangWithEingang vorgang) {
		return this.isForwardable(vorgang) || this.isCollaborationCreateable();
	}

	boolean isForwardable(VorgangWithEingang vorgang) {
		return featureToggleProperties.isForwardByOzgCloudEnabled() && isStatusNeu(vorgang);
	}

	boolean isCollaborationCreateable() {
		return featureToggleProperties.isCollaborationEnabled() && this.userService.hasRole(UserRole.VERWALTUNG_USER);
	}

	private boolean isStatusNeu(VorgangWithEingang vorgang) {
		return vorgang.getStatus() == VorgangStatus.NEU;
	}

	private Link buildSearchOrganisationsEinheitLink() {
		return linkTo(methodOn(OrganisationsEinheitController.class).search(null)).withRel(
				REL_SEARCH_ORGANISATIONS_EINHEIT);
	}
}