/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.clientattribute;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcUpdateClientAttributeRequest;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
@Log4j2
class ClientAttributeRemoteService {
	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private ClientAttributeServiceBlockingStub service;

	void setBooleanReadOnlyClientAttribute(String vorgangId, ClientAttribute clientAttribute, boolean value) {
		try {
			service.update(buildRequest(vorgangId, clientAttribute, value));
		} catch (RuntimeException e) {
			LOG.error("Error on resetting new Postfachnachrichten.", e);
		}
	}

	GrpcUpdateClientAttributeRequest buildRequest(String vorgangId, ClientAttribute clientAttribute, boolean value) {
		return GrpcUpdateClientAttributeRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setAttribute(
						GrpcClientAttribute.newBuilder().setValue(
								GrpcClientAttributeValue.newBuilder().setBoolValue(value).build())
								.setClientName(clientAttribute.getClientName().toString())
								.setAttributeName(clientAttribute.getAttributeName())
								.build())
				.build();
	}
}
