/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import java.util.function.Predicate;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class CommandHelper {

	static final Predicate<Command> IS_DONE = command -> command.getStatus() == CommandStatus.FINISHED
			|| command.getStatus() == CommandStatus.REVOKED || command.getStatus() == CommandStatus.ERROR;

	static final Predicate<Command> IS_PENDING = command -> command.getStatus() == CommandStatus.PENDING
			|| command.getStatus() == CommandStatus.REVOKE_PENDING;

	public static final Predicate<Command> IS_REVOKEABLE = command -> command.getCommandOrder().isRevokeable()
			&& command.getStatus() == CommandStatus.FINISHED;

	public static final Predicate<Command> IS_LOESCHANFORDERUNG = command -> command.getCommandOrder() == CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN
			|| command.getCommandOrder() == CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN;
}