package de.ozgcloud.alfa.common.command;

import java.util.Objects;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;

public abstract class CommandProcessor {

	static final LinkRelation REL_EFFECTED_RESOURCE = LinkRelation.of("effected_resource");
	static final LinkRelation REL_RELATED_RESOURCE = LinkRelation.of("related_resource");

	public void process(EntityModel<Command> model) {
		var command = model.getContent();

		if (Objects.isNull(command)) {
			return;
		}

		model.addIf(CommandHelper.IS_DONE.test(command) && isResponsibleForEffectedResource(command.getCommandOrder()),
				() -> createEffectedResourceLinkBuilder(command).withRel(REL_EFFECTED_RESOURCE));
		model.addIf(isResponsibleForRelatedResource(command.getCommandOrder()),
				() -> createRelatedResourceLinkBuilder(command).withRel(REL_RELATED_RESOURCE));
	}

	public abstract boolean isResponsibleForEffectedResource(CommandOrder order);

	public abstract WebMvcLinkBuilder createEffectedResourceLinkBuilder(Command command);

	public abstract boolean isResponsibleForRelatedResource(CommandOrder order);

	public abstract WebMvcLinkBuilder createRelatedResourceLinkBuilder(Command command);
}