/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.forwarding;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.vorgang.forwarding.ForwardingServiceGrpc.ForwardingServiceBlockingStub;
import de.ozgcloud.vorgang.forwarding.GrpcFindForwardingsRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class ForwardingRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private ForwardingServiceBlockingStub serviceStub;

	@Autowired
	private ForwardingMapper mapper;

	@Autowired
	private ContextService contextService;

	public Stream<Forwarding> findForwardings(String vorgangId) {
		var response = serviceStub.findForwardings(buildFindForwardingsRequest(vorgangId));

		return response.getForwardingsList().stream().map(mapper::fromGrpc);
	}

	private GrpcFindForwardingsRequest buildFindForwardingsRequest(String vorgangId) {
		return GrpcFindForwardingsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setVorgangId(vorgangId)
				.build();
	}
}
