/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Optional;
import java.util.regex.Pattern;

import org.springframework.hateoas.Link;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.resource.OzgcloudResourceURIResolver;

@Component
class VorgangURIResolver implements OzgcloudResourceURIResolver {

	static final String REL_NAME = "vorgang";

	private final Pattern pattern = Pattern.compile(".+/vorgangs/([^/]+)");

	@Override
	public Optional<Link> resolve(String uri) {
		var matcher = pattern.matcher(uri);
		if (!matcher.matches()) {
			return Optional.empty();
		}
		var vorgangId = matcher.group(1);
		return Optional.of(linkTo(VorgangController.class).slash(vorgangId).withRel(REL_NAME));
	}
}
