/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

class FachstelleServiceTest {

	@InjectMocks
	private FachstelleService service;

	@Mock
	private FachstelleRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(FachstelleTestFactory.ID);

			verify(remoteService).getById(FachstelleTestFactory.ID);
		}

		@Test
		void shouldReturnFachstelle() {
			var fachstelle = FachstelleTestFactory.create();
			when(remoteService.getById(FachstelleTestFactory.ID)).thenReturn(fachstelle);

			var gotFachstelle = service.getById(FachstelleTestFactory.ID);

			assertThat(gotFachstelle).isEqualTo(fachstelle);
		}
	}

	@Nested
	class TestSearchFachstellen {

		private final String searchBy = LoremIpsum.getInstance().getWords(5);

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).search(searchBy);
		}

		@Test
		void shouldReturnFachstellen() {
			var fachstelle = FachstelleTestFactory.create();
			when(remoteService.search(searchBy)).thenReturn(Stream.of(fachstelle));

			var fachstellen = callService();

			assertThat(fachstellen).containsExactly(fachstelle);
		}

		private Stream<Fachstelle> callService() {
			return service.searchFachstellen(searchBy);
		}
	}

}