/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import io.grpc.Metadata;
import io.grpc.Status;
import io.grpc.StatusRuntimeException;

class UserServiceTest {

	@InjectMocks
	private UserService service;
	@Mock
	private UserRemoteService remoteService;

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			getById();

			verify(remoteService).getById(UserProfileTestFactory.ID);
		}

		@Test
		void shouldReturnNullOnNotFoundException() {
			when(remoteService.getById(any())).thenThrow(new StatusRuntimeException(Status.NOT_FOUND));

			var userProfile = getById();

			assertThat(userProfile).isNull();
		}

		@SuppressWarnings("null")
		@DisplayName("should throw unavailable exception if exception is other than not found")
		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "NOT_FOUND" })
		void shouldThrowServiceUnavailableException(Status.Code status) {
			when(remoteService.getById(any())).thenThrow(new StatusRuntimeException(Status.fromCode(status), new Metadata()));

			assertThatThrownBy(() -> getById()).isInstanceOf(UserManagerUnavailableException.class);
		}

		private UserProfile getById() {
			return service.getById(UserProfileTestFactory.ID);
		}
	}

	@DisplayName("Get internalId")
	@Nested
	class TestGetInternalId {

		@Test
		void shouldCallRemoteService() {
			when(remoteService.getByExternalId(any())).thenReturn(Optional.of(UserProfileTestFactory.create()));

			getInternalId();

			verify(remoteService).getByExternalId(UserProfileTestFactory.EXTERNAL_ID);
		}

		private Optional<UserId> getInternalId() {
			return service.getInternalId(UserProfileTestFactory.EXTERNAL_ID);
		}
	}
}