package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;

class PostfachNachrichtCommandProcessorTest {

	@Spy
	private PostfachNachrichtCommandProcessor processor;

	@DisplayName("Related resource")
	@Nested
	class TestRelatedResource {

		@DisplayName("is responsible")
		@Nested
		class TestIsResponsible {

			@EnumSource(mode = Mode.INCLUDE, names = { "RECEIVE_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL" })
			@ParameterizedTest(name = "{0}")
			void shouldReturnTrueOnOrder(CommandOrder order) {
				var isResponsible = processor.isResponsibleForRelatedResource(order);

				assertThat(isResponsible).isTrue();
			}

			@EnumSource(mode = Mode.EXCLUDE, names = { "RECEIVE_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL" })
			@ParameterizedTest(name = "{0}")
			void shouldReturnFalseOnOrder(CommandOrder order) {
				var isResponsible = processor.isResponsibleForRelatedResource(order);

				assertThat(isResponsible).isFalse();
			}
		}

		@Test
		void shouldCreateLinkBuilder() {
			var linkBuilder = processor.createRelatedResourceLinkBuilder(CommandTestFactory.create());

			assertThat(linkBuilder).hasToString(PostfachMailController.PATH + "/" + CommandTestFactory.RELATION_ID);
		}
	}

	@DisplayName("Effected resource")
	@Nested
	class TestEffectedResource {

		@DisplayName("is responsible")
		@Nested
		class TestIsResponsible {

			@EnumSource(mode = Mode.INCLUDE, names = { "SEND_POSTFACH_MAIL", "SEND_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL",
					"RECEIVE_POSTFACH_NACHRICHT" })
			@ParameterizedTest(name = "{0}")
			void shouldReturnTrueOnOrder(CommandOrder order) {
				var isResponsible = processor.isResponsibleForEffectedResource(order);

				assertThat(isResponsible).isTrue();
			}

			@EnumSource(mode = Mode.EXCLUDE, names = { "SEND_POSTFACH_MAIL", "SEND_POSTFACH_NACHRICHT", "RESEND_POSTFACH_MAIL",
					"RECEIVE_POSTFACH_NACHRICHT" })
			@ParameterizedTest(name = "{0}")
			void shouldReturnFalseOnOrder(CommandOrder order) {
				var isResponsible = processor.isResponsibleForEffectedResource(order);

				assertThat(isResponsible).isFalse();
			}
		}

		@Test
		void shouldCreateLinkBuilder() {
			var linkBuilder = processor.createEffectedResourceLinkBuilder(CommandTestFactory.create());

			assertThat(linkBuilder).hasToString(PostfachMailController.PATH + "?vorgangId=" + CommandTestFactory.VORGANG_ID);
		}
	}
}